@@ -34,6 +34,7 @@
 #define NO_ITEM_IN_POSITION_SENSORS	13
 #define NO_ITEM_IN_STATUS_CELL 		2
 
+
 struct {
     KeyItem 		*current_keyitem;
     GtkTreeIter 	current_iter;
@@ -49,12 +50,18 @@ enum
     DOT_78  = 78
 };
 
-typedef enum
+const gchar *cursor_style[] =
 {
-    FULLCELL 	= 1,
-    UNDERLINE 	= 2,
-    NOCURSOR	= 0
-}CursorType;    
+    "block",
+    "underline",
+    "user"
+};
+
+const gchar *braille_style[] =
+{
+    "8",
+    "6"
+};
 
 typedef enum
 {
@@ -69,16 +76,11 @@ const struct
     gchar *file_name;
 } trans_table_list [] = 
 {
-    {N_("GERMAN"), 	"GE.A2B"},
-    {N_("US"),		"US.A2B"},
-    {N_("SPAIN"),	"SPAIN.A2B"},
-    {N_("SWEED"),	"SWEED.A2B"},
-    {NULL,	NULL}
-};
-
-const gint cur_value [] = 
-{
-    1,2,0
+    {N_("American English"),	"en_US"},
+    {N_("German"),		"de"},
+    {N_("Spanish"),		"es"},
+    {N_("Swedish"),		"sv"},
+    {NULL,		NULL}
 };
 
 const gint attribute_value [] = 
@@ -92,7 +94,7 @@ const gchar *status_cell_value [] =
     "FontStyle"
 };
 
-
+#define BRLTTY "BRLTTY's BrlAPI"
 /**
  * Braille Settings widgets 					
 **/
@@ -106,6 +108,7 @@ GtkWidget		*w_braille_settings = NULL;
 GtkWidget		*w_braille_device;
 static GtkWidget 	*cb_braille_device;
 static GtkWidget 	*sp_port;
+static GtkWidget	*lb_port;
 
 /**
  * Translation Table widgets 					
@@ -150,7 +153,7 @@ static GtkWidget 	*rb_status_cell [ NO_ITEM_IN_STATUS_CELL ];
 		
 
 /**
- * Positiohn Switches 						
+ * Braille Mapping					
 **/
 static GtkWidget	*w_braille_key_mapping;
 static GtkWidget 	*w_braille_mapping_add_modify;
@@ -159,6 +162,8 @@ static GtkWidget 	*rb_optical_sensor [ NO_ITEM_IN_OPTICAL_SENSORS ];
 static GtkWidget	*tv_braille_keys;
 static GtkWidget	*tv_cmd_list;
 static GtkWidget	*cb_braille_key;
+static GtkWidget	*bt_braille_key_remove;
+static GtkWidget	*bt_braille_key_modify;
 
 extern GList 	*brldev_id_list;
 extern GList	*brldev_description_list;	
@@ -310,7 +315,7 @@ brlui_braille_device_changeing (Braille *braille_setting)
     
     tmp = (gchar*)gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry));
 
-    id = (gchar*) brlconf_device_id_return (tmp);	 
+    id  = (gchar*)brlconf_device_id_return (tmp);	 
     if (id)
 	brlconf_device_set (id);
     braille_setting->port_no = 
@@ -318,6 +323,33 @@ brlui_braille_device_changeing (Braille *braille_setting)
     brlconf_port_no_set (braille_setting->port_no);
 }
 
+static void
+brlui_braille_device_combo_changed (GtkWidget *entry,
+				    gpointer  user_data)
+{
+    gchar   *port_text = NULL;
+    gdouble min_val, max_val;
+    sru_return_if_fail (braille_setting);    
+    
+    if (!strcmp (BRLTTY, gtk_entry_get_text (GTK_ENTRY (entry))))
+    {
+	min_val   = (gdouble)MIN_TTY_CONSOL_NO;
+	max_val   = (gdouble)MAX_TTY_CONSOL_NO;
+	/*To translators: X = X Window System server */
+	port_text = _("X display's _terminal number:");
+    }
+    else
+    {
+    	min_val   = (gdouble)MIN_SERIAL_PORT_NO;
+	max_val   = (gdouble)MAX_SERIAL_PORT_NO;
+    	port_text = _("_Port:");
+    }
+
+    gtk_spin_button_set_range (GTK_SPIN_BUTTON (sp_port), min_val, max_val);
+    gtk_label_set_text (GTK_LABEL (lb_port), port_text);
+    gtk_label_set_use_underline (GTK_LABEL (lb_port), TRUE);
+}
+
 static void
 brlui_settings_response_braille_device (GtkDialog *dialog,
 					gint       response_id,
@@ -355,13 +387,18 @@ brlui_set_handlers_braille_device (GladeXML *xml,
     w_braille_device  = glade_xml_get_widget (xml, "w_braille_device");
     cb_braille_device = glade_xml_get_widget (xml, "cb_braille_device");
     sp_port 	      = glade_xml_get_widget (xml, "sp_port");
-
+    lb_port 	      = glade_xml_get_widget (xml, "lb_port");
 
     g_signal_connect (w_braille_device, "response",
 		      G_CALLBACK (brlui_settings_response_braille_device), 
 		      (gpointer)braille_setting);
     g_signal_connect (w_braille_device, "delete_event",
                       G_CALLBACK (brlui_delete_emit_response_cancel), NULL);        
+
+    glade_xml_signal_connect (xml,"on_combo-entry1_changed",		
+			    GTK_SIGNAL_FUNC (brlui_braille_device_combo_changed));
+
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry), FALSE);
 }
 
 
@@ -412,8 +449,8 @@ brlui_braille_device_value_add_to_widgets (Braille *braille_setting)
 
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cb_braille_device)->entry), 
 			brlconf_return_active_device ());
-    gtk_spin_button_set_value (	GTK_SPIN_BUTTON (sp_port), 
-				(gdouble)braille_setting->port_no);
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON (sp_port), 
+			       (gdouble)braille_setting->port_no);
     return TRUE;
 }
 
@@ -425,10 +462,15 @@ brlui_translation_table_value_add_to_widgets (Braille *braille_setting)
     GtkTreeIter      iter;
     gboolean 	     valid;
     gint 	     l_iter;
+    
 
     sru_return_val_if_fail (w_translation_table, FALSE);
     sru_return_val_if_fail (braille_setting, FALSE);
     
+    g_free (braille_setting->translation_table);
+    braille_setting->translation_table = 
+	    brlconf_translation_table_get ();
+    
     model 	  = gtk_tree_view_get_model ( GTK_TREE_VIEW (tv_translation_table));
     selection     = gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_translation_table));
     
@@ -484,6 +526,22 @@ brlui_translation_table_changeing (Braille *braille_setting)
     g_free (key);
 }
 
+static gboolean
+brlui_search_equal_func (GtkTreeModel  *model,
+			 gint 		column,
+			 const gchar	*key,
+			 GtkTreeIter	*iter,
+			 gpointer	search_data)
+{
+    gchar *list_key = NULL;
+    
+    gtk_tree_model_get (model, 	iter,
+    			column, &list_key,
+                    	-1);
+
+    return g_strcasecmp (key, list_key) > 0; 
+}
+
 
 static GtkTreeModel*
 brlui_create_model (void)
@@ -559,6 +617,13 @@ brlui_set_handlers_translation_table (GladeXML *xml, Braille *braille_setting)
     selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (tv_translation_table));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);    
     
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_translation_table), 
+				    TABLE_COLUMN);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_translation_table),
+					brlui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_translation_table), TRUE);
+
 }
 
 gboolean 
@@ -586,17 +651,14 @@ brlui_load_translation_table (Braille *braille_setting)
 void 
 brlui_braille_style_changeing (Braille *braille_setting)
 {
-    gint style_val;
+    
+    g_free (braille_setting->braille_style);
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rb_braille_style[0])))
-	style_val = DOT_8;
+	braille_setting->braille_style = g_strdup_printf ("%d", DOT_8);
     else
-	style_val = DOT_6;
-	
-    if (style_val != braille_setting->braille_style)
-    {
-	braille_setting->braille_style = style_val;
-	brlconf_style_set (braille_setting->braille_style);
-    }
+    	braille_setting->braille_style = g_strdup_printf ("%d", DOT_6);
+
+    brlconf_style_set (braille_setting->braille_style);
 }
 
 static void
@@ -657,25 +719,20 @@ brlui_load_braille_style (Braille *braille_setting)
 gboolean 
 brlui_braille_style_value_add_to_widgets (Braille *braille_setting)
 {
+    gint iter;
     sru_return_val_if_fail (braille_setting, FALSE);
+    
     if (!w_braille_style)
 	return FALSE;
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_braille_style[0]), TRUE);
     
-    switch (braille_setting->braille_style)
+    for (iter = 0 ; iter < G_N_ELEMENTS (braille_style) ; iter++)
     {
-        case DOT_8:
-	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_braille_style[0]), 
-					TRUE);
-	    break;
-        case DOT_6:
-	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_braille_style[1]), 
-					TRUE);
-	    break;
-        default:   
-	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_braille_style[0]), 
-					TRUE);
-	    break;
+	if (!strcmp (braille_setting->braille_style, braille_style[iter]))
+	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_braille_style[iter]), TRUE);
     }
+
     return TRUE;
 }
 
@@ -683,15 +740,13 @@ static void
 brlui_cursor_setting_changeing (Braille *braille_setting)
 {
     gint i;
-    for (i = 0 ; i < G_N_ELEMENTS (cur_value) ; i++)
+    for (i = 0 ; i < G_N_ELEMENTS (cursor_style) ; i++)
     {
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rb_cursor_settings[i])))
 	{
-	    if (braille_setting->cursor != cur_value[i])
-	    {
-		braille_setting->cursor = cur_value[i];
-		brlconf_cursor_set (braille_setting->cursor);
-	    }
+	    g_free (braille_setting->cursor_style);
+	    braille_setting->cursor_style = g_strdup (cursor_style [i]);
+	    brlconf_cursor_style_set (braille_setting->cursor_style);
 	    break;
 	}
     }
@@ -761,12 +816,13 @@ brlui_cursor_setting_value_add_to_widgets (Braille *braille_setting)
     gint i;
     
     sru_return_val_if_fail (braille_setting, FALSE);
+    
     if (!w_cursor_settings)
 	return FALSE;
     
-    for (i = 0; i < G_N_ELEMENTS (cur_value); i++)
+    for (i = 0; i < G_N_ELEMENTS (cursor_style); i++)
     {
-	if (braille_setting->cursor == cur_value[i])
+	if (!strcmp (braille_setting->cursor_style, cursor_style[i]))
 	{
 	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_cursor_settings[i]),
 					TRUE);
@@ -1145,7 +1201,10 @@ brlui_add_modify_ok (void)
 	return FALSE;
 
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+	gn_show_message (_("No selected command!"));
 	return FALSE;
+    }
     
     if (current_item.current_is_new)
     {
@@ -1165,6 +1224,11 @@ brlui_add_modify_ok (void)
     	    gtk_list_store_append (GTK_LIST_STORE (model), 
 				    &current_item.current_iter);
 	}
+	else
+	{
+	    gn_show_message (_("Invalid device key!"));
+	    return FALSE;
+	}
     }
         
     (current_item.current_keyitem)->command_list =
@@ -1287,8 +1351,10 @@ static void
 brlui_load_braille_key_combination_list (void)
 {
     gint i,j;
-/* This values are hardcoded because, the UI does not get any information
-    from braille device about the button keynames*/
+/* NOTE:
+    This values are hardcoded because, the UI does not get any information
+    from the braille device about the buttons keynames.
+*/
     for (i = 0 ; i < VARIO_DISPLAY_KEY_NO ; i++)
     {
     	braille_key_combination_list = 
@@ -1404,6 +1470,13 @@ brlui_set_handlers_braille_mapping_add_modify (GladeXML *xml)
     
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_cmd_list));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE); 
+    
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_cmd_list), 
+				    CMD_COLUMN);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_cmd_list),
+					brlui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_cmd_list), TRUE);
 
 }
 
@@ -1468,7 +1541,7 @@ brlui_braille_key_mapping_changeing (Braille *braille_setting)
 
 
     removed_items_list =
-	cmdconf_remove_item_from_gconf_list (removed_items_list);
+	cmdconf_remove_items_from_gconf_list (removed_items_list);
 	
     removed_items_list = 
 	brlui_removed_item_list_free (removed_items_list);
@@ -1542,7 +1615,10 @@ brlui_braille_keys_modify (void)
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_braille_keys));
 
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+        gn_show_message (_("No selected item to modify!"));
 	return;
+    }
     
     sru_return_if_fail (GTK_IS_LIST_STORE (model));
     
@@ -1552,7 +1628,7 @@ brlui_braille_keys_modify (void)
 	
     if (!key || strlen (key) == 0)
     {
-	sru_message (_("brlui:Invalid selected item"));	    
+	sru_message (_("Invalid selected item!"));	    
 	g_free (key);
 	return;
     }
@@ -1619,8 +1695,11 @@ brlui_remove_keyitem_from_list (GSList 	*list,
     GtkTreeSelection *selection = gtk_tree_view_get_selection (tree_view);	    
     
     if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+        gn_show_message (_("No selected item to remove!"));
 	return list;
-	    
+    }	    
+    
     sru_return_val_if_fail (GTK_IS_LIST_STORE (model), list);
     
     gtk_tree_model_get (model, &iter,
@@ -1629,7 +1708,7 @@ brlui_remove_keyitem_from_list (GSList 	*list,
 	
     if (!key || strlen (key) == 0)
     {
-        sru_message (_("brlui:Invalid selected item"));
+        sru_message (_("Invalid selected item!"));
         g_free (key);
         return list;
     }
@@ -1669,6 +1748,18 @@ brlui_braille_keys_remove_clicked (GtkWidget *button,
 
 }
 
+static void
+brlui_braille_keys_selection_changed (GtkTreeSelection *selection,
+				      gpointer  	user_data)
+{
+    gboolean 	sensitive;
+    
+    sensitive = gtk_tree_selection_get_selected (selection, NULL, NULL);
+    
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_braille_key_remove), sensitive);
+    gtk_widget_set_sensitive (GTK_WIDGET (bt_braille_key_modify), sensitive);
+}
+
 
 static GtkListStore*
 brlui_populate_list_store_with_keys (GtkListStore *store, 
@@ -1731,6 +1822,9 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
     gint i;
     gchar *name;
     w_braille_key_mapping = glade_xml_get_widget (xml, "w_braille_mapping");
+    bt_braille_key_modify = glade_xml_get_widget (xml, "bt_braille_key_modify");
+    bt_braille_key_remove = glade_xml_get_widget (xml, "bt_braille_key_remove");
+    tv_braille_keys	  = glade_xml_get_widget (xml, "tv_braille_keys");
     
     for (i = 0 ; i < NO_ITEM_IN_POSITION_SENSORS ; i++)
     {
@@ -1748,9 +1842,6 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
 	name = NULL;
     }
 
-    tv_braille_keys	= glade_xml_get_widget (xml, "tv_braille_keys");
-
-
     glade_xml_signal_connect (xml,"on_bt_braille_key_remove_clicked",  
 		GTK_SIGNAL_FUNC (brlui_braille_keys_remove_clicked));
 
@@ -1769,6 +1860,11 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
     
     gtk_tree_view_set_model (GTK_TREE_VIEW (tv_braille_keys), model);
     
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_braille_keys));
+    
+    g_signal_connect (selection, "changed",
+		      G_CALLBACK (brlui_braille_keys_selection_changed), NULL);
+    
     gtk_tree_sortable_set_sort_column_id ( GTK_TREE_SORTABLE (model), 
 					    KEYS_COLUMN, 
 					    GTK_SORT_ASCENDING);
@@ -1794,8 +1890,16 @@ brlui_set_handlers_braille_key_mapping	(GladeXML *xml,
 							NULL);    
     gtk_tree_view_append_column (GTK_TREE_VIEW (tv_braille_keys), column);
         
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv_braille_keys));
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+    
+    brlui_braille_keys_selection_changed (selection, NULL);
+    
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tv_braille_keys), 
+				    KEYS_COLUMN);    
+    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tv_braille_keys),
+					brlui_search_equal_func,
+					NULL, NULL);
+    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv_braille_keys), TRUE);
 }
 
 
