@@ -612,7 +612,8 @@ gs_speak (SRSVoice 	*voice,
 	  SRSText 	*text )
 {
     GSSpeaker *gss;
-
+    int rv = 0;
+    
     /* sel current speaker for voice->TTSVoiceName */
     gss = gs_speakers_select_speaker (voice->tts_voice_name);
     if (gss)
@@ -672,14 +673,14 @@ gs_speak (SRSVoice 	*voice,
 		    current_speaker);
 #endif
     	    /* resume_text = text->Text; */
-    	    GNOME_Speech_Speaker_say (current_speaker, 
+    	    rv = GNOME_Speech_Speaker_say (current_speaker, 
 				      text->text, 
 				      gs_ev () );
 
-	    if (BONOBO_EX (&ev) )
-	    {
+	    if (BONOBO_EX (&ev))
 		gs_check_ev (GS_ERR_GENERIC, __LINE__);
-	    }
+	    if (rv == -1)
+	    	sru_warning("\"%s\" has crashed.", voice->tts_voice_name);
         }
     }	
 }
