@@ -121,7 +121,7 @@ typedef GList SRWElements;
 
 #define SRW_DELTA_BASELINE_THRESHOLD	4
 #define SRW_BASELINE_RATIO		0.66
-
+#define SRW_MANAGE_DESCENDATS_STEP	5
 /*__________________________< DEBUG>__________________________________________*/
 #undef	SRW_COMPOSITE_DEBUG 
 #undef	SRW_BENCHMARKING
@@ -135,6 +135,7 @@ typedef GList SRWElements;
 #undef	SRW_CLIPPING_DEBUG
 #undef	SRW_CLIPPING_DEBUG
 #undef	SRW_ACC_LINE_CACHE	    
+#undef	SRW_MENAGE_DESCENDANTS
 /*__________________________</DEBUG>__________________________________________*/
 
 /*______________________________< MACROS>_____________________________________*/
@@ -820,7 +821,9 @@ srw_text_chunk_from_accessible (Accessible	*accessible,
 		    role == SPI_ROLE_LIST_ITEM 		||
 		    role == SPI_ROLE_LABEL 		||
 		    role == SPI_ROLE_COLUMN_HEADER 	||
-		    role == SPI_ROLE_TABLE_COLUMN_HEADER||
+		    role == SPI_ROLE_TABLE_COLUMN_HEADER||		    
+		    role == SPI_ROLE_TABLE_ROW_HEADER 	||
+		    role == SPI_ROLE_TABLE_CELL		||
 		    role == SPI_ROLE_DIALOG 		||
 		    role == SPI_ROLE_COLOR_CHOOSER	||	
 		    role == SPI_ROLE_PAGE_TAB 		||
@@ -2193,7 +2196,7 @@ srw_elements_from_accessible (Accessible 	*accessible,
 	AccessibleComponent 	*component,
 				*child_component;
 	AccessibleStateSet	*states;
-	SRWBoundaryRect 	bounds;
+	SRWBoundaryRect 	bounds, child_bounds;
 	SRWBoundaryRect		**clip_bounds;
 	SRWTextChunk 		*text_chunk;	
 	int 			n_children, 
@@ -2208,7 +2211,8 @@ srw_elements_from_accessible (Accessible 	*accessible,
 
 	/*SELECTED objects don't have the SHOWING state, but they are visible*/
 	if (AccessibleStateSet_contains (states, SPI_STATE_SHOWING)  ||
-	    AccessibleStateSet_contains (states, SPI_STATE_SELECTED)  ) 
+	    AccessibleStateSet_contains (states, SPI_STATE_SELECTED)  ||
+	    AccessibleStateSet_contains (states, SPI_STATE_TRANSIENT) ) 
 	{
 	    if (Accessible_isComponent (accessible)) 
 	    {
@@ -2276,9 +2280,11 @@ srw_elements_from_accessible (Accessible 	*accessible,
 						}
 						if (!text_chunk->clip_bounds.is_empty && 
 						    text_chunk->text_bounds.height > 0)
+						{
 						    elements = g_list_insert_sorted (elements, 
 										text_chunk, 
 										srw_elements_compare_text_chunk_y);
+						}
 						else
 						    srw_text_chunk_free (text_chunk);					
 					}
@@ -2299,22 +2305,147 @@ srw_elements_from_accessible (Accessible 	*accessible,
 	 */
 	    n_children = Accessible_getChildCount (accessible);
 	    parent_selected = AccessibleStateSet_contains (states, SPI_STATE_SELECTED) || 
-			      (!AccessibleStateSet_contains (states, SPI_STATE_SELECTABLE) &&
-			       AccessibleStateSet_contains (states, SPI_STATE_SHOWING));
+	        	(!AccessibleStateSet_contains (states, SPI_STATE_SELECTABLE) &&
+		          AccessibleStateSet_contains (states, SPI_STATE_SHOWING));
 	    parent_layer = layer;
-	    for (child_n = 0; child_n < n_children; ++child_n) 
+	    if (!AccessibleStateSet_contains (states, SPI_STATE_MANAGES_DESCENDANTS))
 	    {
-		child = Accessible_getChildAtIndex (accessible, child_n);
-		
-		srw_elements_from_accessible (child, 
-					     focused_accessible,
-					      clip_bounds,
-					      parent_selected,
-					      parent_layer); 
-		Accessible_unref (child);
-	    }
 
-	}
+		for (child_n = 0; child_n < n_children; ++child_n) 
+		{
+		    child = Accessible_getChildAtIndex (accessible, child_n);
+		    srw_elements_from_accessible (child, 
+						  focused_accessible,
+						  clip_bounds,
+						  parent_selected,
+						  parent_layer); 
+		    Accessible_unref (child);
+		}
+	    }
+	    else
+	    {
+		int x, y, step_y;
+		gboolean support_at_point = FALSE;
+		GList 	*children = NULL,  *iter = NULL;
+
+		component = Accessible_getComponent (accessible);
+		x = bounds.x ;
+		y = bounds.y ;
+
+		child = AccessibleComponent_getAccessibleAtPoint 
+				(component,
+			        bounds.x + bounds.width/2,
+				bounds.y + bounds.height/2,
+				SPI_COORD_TYPE_SCREEN);
+		if (child)
+		{
+			Accessible_unref (child);
+			support_at_point = TRUE;
+		}				
+		else
+		{
+		    sru_warning ("AccessibleComponent_getAccessibleAtPoint  is NOT working");		    			    			
+		}
+#ifdef SRW_MANAGE_DESCENDENTS
+		sru_message ("parent [%d, %d, %ld, %ld] ", x, y, bounds.width, bounds.height);
+#endif
+		while (support_at_point && component && y < bounds.y + bounds.height )
+		{
+		    step_y = -1;
+		    while (x < bounds.x + bounds.width)
+		    {
+			child = AccessibleComponent_getAccessibleAtPoint 
+				(component,
+			        x,
+				y,
+				SPI_COORD_TYPE_SCREEN);
+			if (child)
+			{
+#ifdef SRW_MANAGE_DESCENDENTS			
+			    sru_message ("Manage descendents - child %p %s", child,  Accessible_getName (child));		    			    
+			    sru_message ("next column :  %d %d", x,y);
+#endif			
+			    child_component =  Accessible_getComponent (child);
+			    if (child_component)
+			    {
+				AccessibleComponent_getExtents (child_component,
+							&child_bounds.x,
+							&child_bounds.y,
+							&(child_bounds.width),
+							&(child_bounds.height),
+							SPI_COORD_TYPE_SCREEN);	
+#ifdef SRW_MANAGE_DESCENDENTS			
+				sru_message ("child [%ld, %ld, %ld, %ld] ", 
+				    child_bounds.x, 
+				    child_bounds.y,
+				    child_bounds.width, 
+				    child_bounds.height);													    
+#endif
+				Accessible_unref (child_component);
+				if (step_y == -1)
+				{
+			    	    step_y = child_bounds.height;
+#ifdef SRW_MANAGE_DESCENDENTS			
+				    sru_message ("step_y = %d",step_y);
+#endif
+				}
+				if ( child_bounds.x > 0)
+			    	    x = child_bounds.x + child_bounds.width;			    
+				else
+				{
+			    	    x += SRW_MANAGE_DESCENDATS_STEP;
+				    Accessible_unref (child);			    		    
+				    break;
+				}
+				children = g_list_prepend (children, child);
+			} /*if (child_component)*/
+
+			x += SRW_MANAGE_DESCENDATS_STEP; 
+		    }/*if (child)*/
+		    else
+		    {
+			static int break_cond = 20;
+			if (step_y < 0) 
+			    step_y = SRW_MANAGE_DESCENDATS_STEP;
+			if (--break_cond < 0)
+			{
+			    break;
+			}
+			else
+			{
+				x += SRW_MANAGE_DESCENDATS_STEP;
+#ifdef SRW_MANAGE_DESCENDENTS							
+				sru_message ("Manage descendents - NO child %p", child);		    			    
+				sru_message ("Point %d %d", x,y);
+#endif
+			}
+		    }		    
+		    }/*while x*/
+		y += step_y;
+		x = bounds.x ;
+#ifdef SRW_MANAGE_DESCENDENTS			
+		sru_message ("next line: Point %d %d | step_y %d", x,y, step_y);
+#endif
+		}/*while y*/
+	    Accessible_unref (component);
+
+	    for (iter = g_list_first (children); iter; iter = iter->next) 
+	    {
+		child = (Accessible *) iter->data;
+
+		srw_elements_from_accessible (child, 
+						  focused_accessible,
+						  clip_bounds,
+						  parent_selected,
+						  parent_layer); 
+		Accessible_unref (child);			    			    
+
+	    }
+	    g_list_free (children);
+	}/*else - MD*/
+
+	    
+	}/*if states*/
 
 	AccessibleStateSet_unref (states);
 
